#include "NoisyAdaptedPSO.h"
#include <time.h>
#include <stdlib.h>

//This function calculates the W in the screen to the best procedure
double CalculateW(double sampleVariance1,double sampleVariance2,double alpha,int numberOfObservations,int numberOfParticles)
{
	double w;
	w=pow((pow(tinv(pow(1-alpha,(double)1/(numberOfParticles-1)),numberOfObservations-1),2)*sampleVariance1/numberOfObservations)+(pow(tinv(1-alpha,numberOfObservations-1),2)*sampleVariance2/numberOfObservations),0.5);
	return w;
}

//This function calculates the distance
double CalculateDistance(vector<double> &v1,vector <double> &v2)
{
	double distance=0;
	for(int i=0;i<v1.size();i++)
	{
		distance+=pow(v1[i]-v2[i],2);
	}
	distance=sqrt(distance);
	return distance;
}


//Choosing the globals bests using the screen to the best procedure
vector<int> ChooseGlobalBests(Particle particles[],vector<vector<double>> &observations,int numberOfParticles,double alpha)
{
	
	vector<int> globalBestIndex;
	vector<int> finalGlobalBestIndex;
	vector<int> temp;
	double xi;
	double si;
	double xl;
	double sl;
	int flag=0;
	int flag2=0;
	int flag3=0;
	int flag4=0;
	double winner=0;
	for(int i=0;i<numberOfParticles;i++)
	{
		flag=0;
		flag2=0;
		//These are the requirements to use the screen to the best procedure
		xi=CalculateSampleMean(observations[i]);
		si=CalculateSampleVariance(observations[i]);
		for(int j=0;j<numberOfParticles;j++)
		{
			if(i!=j) 
			{
				xl=CalculateSampleMean(observations[j]);
				sl=CalculateSampleVariance(observations[j]);
				if(xi>xl+CalculateW(si,sl,alpha,observations[i].size(),numberOfParticles))
				{
					flag=1;
				}
			}
		}
		for(int k=0;k<globalBestIndex.size();k++)
			{

				if(particles[globalBestIndex[k]].getEntireLBest()==particles[i].getEntireLBest())
				{
					flag2=1;
				}
			}
		if(flag==0 & flag2==0)
		{
			globalBestIndex.push_back(i);
		}
	}
	/*for(int i=0;i<globalBestIndex.size();i++)
	{
		winner=globalBestIndex[i];
		for(int j=i+1;j<globalBestIndex.size();j++)
		{

		//For Quadratic 100
			//Griewank 300
			//goldsteinPrice 50,,,,0.1
		if(CalculateDistance(particles[globalBestIndex[i]].getEntireLBest(),particles[globalBestIndex[j]].getEntireLBest())<400)
		{
			temp.push_back(globalBestIndex[j]);
		}
		}
		for(int k=0;k<temp.size();k++)
		{
			if(CalculateSampleMean(observations[temp[k]])<CalculateSampleMean(observations[winner]))
			{
				winner=temp[k];
			}
		}
		for(int k=0;k<finalGlobalBestIndex.size();k++)
		{
			if(finalGlobalBestIndex[k]==winner)
				flag3=1;
		}
		if(flag3==0)
		{
			finalGlobalBestIndex.push_back(winner);
		}
		flag3=0;
		temp.clear();
		temp.resize(0);

	}*/


	return globalBestIndex;
}

//Calculating the Quality factor using the procedure mentioned in the paper
vector<double> CalculateQ(Particle particles[],vector<int> &globalBestIndex,vector<vector<double>> &observations)
{
	vector<double> Q;
	int w=0;
	vector<double> noisyObjectiveValues;
	double max=0;
	for(int i=0;i<globalBestIndex.size();i++)
	{
		noisyObjectiveValues.push_back(particles[globalBestIndex[i]].getlBestValue());
	}
	
	for (int i=1;i<globalBestIndex.size();i++)
	{
		if(noisyObjectiveValues[i]>noisyObjectiveValues[w])
		{
			w=i;
		}
	}
	for(int i=0;i<globalBestIndex.size();i++)
	{
		//Q.push_back((noisyObjectiveValues[w]-noisyObjectiveValues[i])/abs(noisyObjectiveValues[w]));
		Q.push_back(noisyObjectiveValues[w]/noisyObjectiveValues[i]);
	}
	double temp=0;
	for(int i=0;i<Q.size();i++)
	{
		
			if(Q[i]<temp)
			{
				temp=Q[i];
			}
		
	}
	if(temp<0)
	{
		for(int i=0;i<Q.size();i++)
	{
			
				Q[i]-=temp;
			
		
	}
	}
	for(int i=0;i<Q.size();i++)
	{
		if(Q[i]>max)
		{
		max=Q[i];
		}
	}
	for(int i=0;i<Q.size();i++)
	{
		if(max!=0)
		{
		Q[i]/=max;
		}
	}
	return Q;
}

double calculateSumOfLoyaltyFactors(vector<double> &loyaltyFactor)
{
	double sum=0;
	for(int i=0;i<loyaltyFactor.size();i++)
	{
		sum+=loyaltyFactor[i];
	}
	return sum;
}

//This function findes the maximum of a vector
double FindMaximumDistance(vector<double> &distance)
	{
		double max=0;
		for(int i=0;i<distance.size();i++)
		{
			
				if(distance[i]>max)
				{
					max=distance[i];
				}
			
		}
		return max;
	}
//This function divides every elements of a vector by the maximum of it
void NormalizeDistance(vector<vector<double>> &distance)
{
	double max;
	for(int i=0;i<distance.size();i++)
		{
			max=0;
			for(int j=0;j<distance[i].size();j++)
			{
				if(distance[i][j]>max)
				{
					max=distance[i][j];
				}
				
			}
			for(int j=0;j<distance[i].size();j++)
			{
				if(max!=0)
				{
				distance[i][j]/=max;
				}
			}
		}
}

//This function calculates the tendency factor according to what already mentioned in the paper
vector<vector<double>> CalculateT(vector<vector<double>> &distance,vector<double> &qualityFactor,double QDRatio,Particle particles[],vector<int> globalBestIndex)
{
	double temp=0;
	vector<vector<double>> t;
	for(int i=0;i<distance.size();i++)
	{
		t.push_back(vector<double>());
		for(int j=0;j<distance[i].size();j++)
		{
			t[i].push_back(-1*distance[i][j]+QDRatio*qualityFactor[j]);
			/*if(particles[i].getGlobalBest()==particles[globalBestIndex[j]].getEntireLBest())
			{
				t[i][j]=+loyaltyC;
			}*/

		}
	}
	for(int i=0;i<t.size();i++)
	{
		for(int j=0;j<t[i].size();j++)
		{
			if(t[i][j]<temp)
			{
				temp=t[i][j];
			}
		}
	}
	if(temp<0)
	{
		for(int i=0;i<t.size();i++)
	{
		for(int j=0;j<t[i].size();j++)
		{
			
				t[i][j]-=temp;
			
		}
	}
	}
	return t;

}

//This function derives the probability from tendency factor
void CalculateP(vector<vector<double>> &t)
{
	double sum=0;
	for(int i=0;i<t.size();i++)
	{
		sum=calculateSumOfLoyaltyFactors(t[i]);
		for(int j=0;j<t[i].size();j++)
		{
			t[i][j]/=sum;
		}
	}
	
}
//Calculating the maximum element of a vector
int CalculateMax(vector<double> &vector)
{
	int max=0;
	for(int i=0;i<vector.size();i++)
	{
		if(vector[i]>vector[max])
		{
			max=i;
		}

	}
	return max;
}

//This function assigns the global best with largest probability to the particle
void AssignGlobalBest(vector<vector<double>> &probabilityM,Particle particles[],vector<int> &globalBests)
{
	int maxP;
	for(int i=0;i<probabilityM.size();i++)
	{
		maxP=CalculateMax(probabilityM[i]);
		particles[i].setGlobalBest(particles[globalBests[maxP]].getEntireLBest());
	}
}

//This function finds the best position in the swarm and reports as the global best (with noise)
int FindBest(Particle particles[],int numberOfParticles)
{
	int best=0;
	for(int i=1;i<numberOfParticles;i++)
	{
		if(particles[i].getlBestValue()<particles[best].getlBestValue())
		{
			best=i;
		}
	}
	return best;
}
vector<int> globalBestCluster(vector<vector<double>> &observations,Particle Particles[],int dimension,int numberOfCluster,vector<int> globalBestIndex)
{
	vector<int> clusterIndex;
	vector<vector<double>> m;
	vector<vector<int>> clusterMembers;
	vector<int> clusteredGlobalBests;
	double sum;
	double min;
	int flag=1;
	double best=0;
	for(int i=0;i<numberOfCluster;i++)
	{
		m.push_back(vector<double>());
	}
	for(int i=0;i<numberOfCluster;i++)
	{
		clusterMembers.push_back(vector<int>());
	}
	
	//srand(time(NULL));
	int r=0;
	for(int i=0;i<globalBestIndex.size();i++)
	{
		if(r==numberOfCluster)
		{
			r=0;
		}
		//cout<<r<<endl;
		//******************************************
		clusterIndex.push_back(r);
		r++;
	}
	for(int j=0;j<clusterIndex.size();j++)
	{
		clusterMembers[clusterIndex[j]].push_back(globalBestIndex[j]);
	}
	
	for(int i=0;i<numberOfCluster;i++)
	{
		
		for(int k=0;k<dimension;k++)
		{
			sum=0;
			
			for(int j=0;j<clusterMembers[i].size();j++)	
			{
				sum+=Particles[clusterMembers[i][j]].getElements(k);
				
			}
			m[i].push_back(sum/clusterMembers[i].size());
		}
	}
	
	while(flag)
	{
		flag=0;
		for(int i=0;i<globalBestIndex.size();i++)
		{
			min=0;
			for(int j=1;j<numberOfCluster;j++)
			{
				if(CalculateDistance(Particles[globalBestIndex[i]].getEntireElement(),m[j])<CalculateDistance(Particles[globalBestIndex[i]].getEntireElement(),m[min]))
				{
					min=j;
				}
			}
			if(clusterIndex[i]!=min)
			{
				clusterIndex[i]=min;
				flag=1;
			}
		}
		for(int i=0;i<numberOfCluster;i++)
		{
		
			for(int k=0;k<dimension;k++)
				{
					sum=0;
					for(int j=0;j<clusterMembers[i].size();j++)	
					{
						sum+=Particles[clusterMembers[i][j]].getElements(k);
					}
					m[i][k]=sum/clusterMembers[i].size();
				}
		}
	}

	for(int i=0;i<numberOfCluster;i++)
	{
		best=0;
		for(int j=0;j<clusterMembers[i].size();j++)
		{
			if(CalculateSampleMean(observations[clusterMembers[i][j]])<CalculateSampleMean(observations[clusterMembers[i][best]]))
			{
				best=j;
			}
		}
		clusteredGlobalBests.push_back(clusterMembers[i][best]);
	}
	return clusteredGlobalBests;



}