#include"PSOFunctions.h"
#include "RandomVariates.h"
#include<vector>
#include"NoisyAdaptedPSO.h"
#include<string>
#include <sstream>


int main()
{
	/*
	*****************This code is for PSO-sg*******************

	In order to run the code, one first should write his/her own test function in "double objectiveValueWithoutNoise(vector<double> &newVector,int functionNumber)" function existing in the "ContinuousParticle.cpp".
	Right now, the ackley function is used as a sample to show how to write the function. Please follow the same procedure to write your own function.
	In the following lines, you can change and set the parameters you need to use for the algorithm. The explanations about each parameter is written above it.
	Afte setting the test function and parameters, the code is ready to run by pressing "ctrl+F5"

	The output of the algorithm is printed into a csv file named "results.csv". The output contains the mean and variance of the performance of algorithm at each iteration.

	*/

	/*********************************************************
	**********************************************************/
	//The dimension of test function should be set here
	int dimension=10;

	//Number of Macro-replications of PSO can be set here
	int numberOfIterations=100;

	//The variance of the guassian noise can be set here (The mean is 0)
	double noiseVariance=4;

	//Number of particles can be set here (default for the experiments is 50)
	const int numberOfParticles=50;

	//you can change the PSO parameters here
	double Budget=200; //The maximum number of PSO iterations we want to run at each iteration

	//PSO parameters
	double w=0.729,c1=1.49445,c2=1.49445, VMax=10;//w, c1,c2 are the actual parameters for PSO. VMax is the max velocity which can be set according to the test function

	//The number of observations on each particle:
	int numberOfObservations=5;

	//Number of Clusters for PSO-sg should be entered here (Default is 5)
	int numberOfCluster=5;

	//Quality distance ratio should be entered here (default is 0.25)
	double QDRatio=0.25;
	/*********************************************************
	*********************************************************/

	//No changes are needed from here to the end of the code
	
	
	
	double a=0;
	double b0=0;
	ofstream myfile;
	ofstream myfile2;
	vector<int> v;
	vector<vector<double>> distance;
	vector<vector<double>> loyaltyFactor;
	vector<vector<double>> t;
	vector<vector<double>> probabilityMatrix;
	vector<vector<double>> observations;
	int temp;
	int flag=0;
	double sum=0;
	Particle particles[numberOfParticles];
	ifstream input;
	double temp3;
	vector<vector<double>> initial;
	//In order to have the same initial random numbers, we read the random numbers from an input file we have genereted beforehand
	input.open("random.txt");
	for(int i=0;i<numberOfParticles;i++)
	{
		initial.push_back(vector<double>());
		observations.push_back(vector<double>());
		for(int j=0;j<dimension;j++)
		{
			input>>temp3;
			initial[i].push_back(temp3);
		}
	}
	input.close();
	
	vector<double> Q;
	vector<vector<double>> results;
	vector<double> tempVector;
	vector<double> tempGbest;
	vector<vector<double>> lBestObservations;

	
	for(int p=0;p<numberOfIterations;p++)
	{
		cout<<"iteration: "<<p+1<<endl;
		//This vector saved the results
		results.push_back(vector<double>());
		w=0.5,c1=1,c2=1, VMax=100;
		//This loop is for using the PSO iteratively in each macroreplication
	for(int l=0;l<Budget;l++)
	{
		
		observations.clear();
		observations.resize(0);
		observations.resize(numberOfParticles);
		lBestObservations.clear();
		lBestObservations.resize(0);
		lBestObservations.resize(numberOfParticles);
		if(l!=0)
		{
		for(int i=0;i<numberOfParticles;i++)
		{
			a=objectiveValueWithoutNoise(particles[i].getEntireLBest());
			for(int j=0;j<numberOfObservations;j++)
			{	//regular experiments:
				lBestObservations[i].push_back(a+Normal(0,noiseVariance));

				//proportional experiment:
				//lBestObservations[i].push_back(a+Normal(0,sqrt(a-b0)));
			}
			particles[i].setlBestValue(CalculateSampleMean(lBestObservations[i]));
		}
		}
		
		
		
		
		//if we are in the first iteratin we should initialize the particles
		if(l==0)
		{
		
		for(int r=0;r<numberOfParticles;r++)
	{
		
		particles[r].initializeVector(initial[r]);
	}
		for(int i=0;i<numberOfParticles;i++)
		{
			a=objectiveValueWithoutNoise(particles[i].getEntireElement());
			
			for(int j=0;j<numberOfObservations;j++)
			{
				srand(time(NULL));

				//regular noise
				observations[i].push_back(a+Normal(0,noiseVariance));

				//proportional noise
				//observations[i].push_back(a+Normal(0,sqrt(a-b0)));
			}
		}
		for(int i=0;i<numberOfParticles;i++)
		{
			particles[i].setlBestValue(CalculateSampleMean(observations[i]));
		}
		
		tempGbest=findGlobalBest(particles,numberOfParticles);
		for(int j=0;j<numberOfParticles;j++)
	{
		
		
			particles[j].setGlobalBest(tempGbest);
			
	}
		lBestObservations=observations;
		
		}
		else
		{
			for(int i=0;i<numberOfParticles;i++)
			{
				observations.push_back(vector<double>());
				a=objectiveValueWithoutNoise(particles[i].getEntireElement());
				for(int j=0;j<numberOfObservations;j++)
				{
					//regular noise
					observations[i].push_back(a+Normal(0,noiseVariance));

					//proportional
					//observations[i].push_back(a+Normal(0,sqrt(a-b0)));
				}
			}
		}
		v.resize(0);
		distance.resize(0);
		probabilityMatrix.resize(0);
		t.resize(0);
		loyaltyFactor.resize(0);
		for(int j=0;j<numberOfParticles;j++)
			{
				FindLocalBest(particles[j].getEntireElement(),particles[j],observations[j]);
				
				
			}
		//The process of choosing candidate global bests happens here
		v=ChooseGlobalBests(particles,lBestObservations,numberOfParticles,0.05);
		
 
  
		
		
  //clustering part::removed if commented out
		if(v.size()>numberOfCluster)
		{
			v=globalBestCluster(lBestObservations, particles,dimension,numberOfCluster,v);
			
		}
		
		
	
		//This for loop calculates the distance between each particle and the global bests
	for(int i=0;i<numberOfParticles;i++)
	{
		flag=0;
		distance.push_back(vector<double>());
		for(int j=0;j<v.size();j++)
		{
			distance[i].push_back(CalculateDistance(particles[i].getEntireElement(),particles[v[j]].getEntireLBest()));
		}
		
	}
	//We normalize the distance here
	NormalizeDistance(distance);
	//Quality factor will be calculated here
	Q=CalculateQ(particles,v,observations);
  
  
	//calculating tendency factor
	t=CalculateT(distance,Q,QDRatio,particles,v);
	//calculating probability
	CalculateP(t);
	//assigning the global bests
	AssignGlobalBest(t,particles,v);
	
	
	//Doing the traditional PSO using multipe global bests
	PSO(particles,Budget,numberOfParticles,observations,dimension,w,c1,c2,VMax);
	results[p].push_back(objectiveValueWithoutNoise(particles[FindBest(particles,numberOfParticles)].getGlobalBest()));
	
	}
	}
//We print the output in a csv file 
  myfile.open ("results.csv");
  myfile<<"Iteration"<<","<<"Mean"<<","<<"Variance"<<"\n";
	  for(int j=0;j<Budget;j++)
	  {
		  tempVector.clear();
		  tempVector.resize(0);
		  for(int i=0;i<numberOfIterations;i++)
		  {
			  tempVector.push_back(results[i][j]);
		  }
		  //Sample mean and sample variance will be saved in results.csv
		  myfile<<j+1<<","<<CalculateSampleMean(tempVector)<<","<<CalculateSampleVariance(tempVector)<<"\n";
	  }
  myfile.close();
	
	cin.get();
	

	return 0;
}
