#include "ContinuousParticle.h"
#include"PSOFunctions.h"
#include"RandomVariates.h"

//This Function Performs the OCBA procedure
void OCBA(Particle particles[],vector<vector<double>> &observations,int totalBudget,int numberOfParticles,double noiseVariance,int delta,int functionNumber)
{
	//The number of initial sampling for the OCBA
	int n0=3;
	int best=0;
	double a=0;
	double b0=0;
	double sum=0;
	double finalSummation=0;
	double T=numberOfParticles*n0;
	int addN=0;
	int gBestIndex=findGlobalBestIndex(particles,numberOfParticles);
	Particle temp;
	vector<double> coefficients;
	vector<int>numberOfObservations;
	numberOfObservations.resize(numberOfParticles);
	observations.clear();
	observations.resize(numberOfParticles);
	
	coefficients.resize(numberOfParticles);
	for(int j=0;j<numberOfParticles;j++)
	{
		a=objectiveValueWithoutNoise(particles[j].getEntireElement());
	for(int i=0;i<n0;i++)
	{
		//regular
		observations[j].push_back(a+Normal(0,noiseVariance));

		//proportional

		//observations[j].push_back(a+Normal(0,sqrt(a-b0)));
	}
	}
	T+=delta;
	while(T<totalBudget)
	{
		finalSummation=0;
		for(int i=0;i<numberOfParticles;i++)
		{
			coefficients[i]=CalculateSampleVariance(observations[i])/pow((CalculateSampleMean(observations[i])-particles[gBestIndex].getlBestValue()),2);
		}
	
	
	for(int i=0;i<numberOfParticles;i++)
	{
		finalSummation+=coefficients[i];
	}

	for(int i=0;i<numberOfParticles;i++)
	{
		numberOfObservations[i]=coefficients[i]*T/finalSummation;
	}

	for(int i=0;i<numberOfParticles;i++)
	{
		addN=numberOfObservations[i]-observations[i].size();
		if(addN>0)
		{
			a=objectiveValueWithoutNoise(particles[i].getEntireElement());
			for(int j=0;j<addN;j++)
			{	
				//regular
				observations[i].push_back(a+Normal(0,noiseVariance));

				//proportional
				//observations[i].push_back(a+Normal(0,sqrt(a-b0)));
			}
		}
	}
	T+=delta;
	}

}


//Generating a random number between low and high
double RandomNumber(int low, int high)
	{
		return low + ((high - low) + 1) * rand() / (RAND_MAX + 1.0); 
	}

//This function fins the best position in entire swarm
vector<double> findGlobalBest(Particle particles[],int numberOfParticles)
	{
		//setting a default value (infinity) for min objective function
		int gBest=0;
		for(int i=1;i<numberOfParticles;i++)
		{
			if(particles[i].getlBestValue()<particles[gBest].getlBestValue())
			{
				gBest=i;
			}
		}
		return particles[gBest].getEntireLBest();
	}
int findGlobalBestIndex(Particle particles[],int numberOfParticles)
	{
		//setting a default value (infinity) for min objective function
		int gBest=0;
		for(int i=1;i<numberOfParticles;i++)
		{
			if(particles[i].getlBestValue()<particles[gBest].getlBestValue())
			{
				gBest=i;
			}
		}
		return gBest;
	}

//The procedure of updating the velocity in PSO algorithm
void updateVelocity(Particle &particles,int dimension,double w,double c1,double c2,int V_Max)
	{

		vector<double> newVelocity(dimension);
		for(int i=0;i<dimension;i++)
		{
			newVelocity[i]=w*(particles.getVelocity(i)+c1*RandomNumber(0,1)*(particles.getLbest(i)-particles.getElements(i)))+c2*RandomNumber(0,1)*(particles.getGlobalBest()[i]-particles.getElements(i));
			if(abs(newVelocity[i])>V_Max)
			{
				if(newVelocity[i]<0)
				{
					newVelocity[i]=-1*V_Max;
				}
				else
				{
				newVelocity[i]=V_Max;
				}
			}
			particles.setVelocity(i,newVelocity[i]);
			particles.setElements(i,particles.getElements(i)+newVelocity[i]);
		}
		
	}

//The PSO algorithm
void PSO(Particle particles[],int budget,int numberOfParticles,vector<vector<double>> &observations,int dimension,double w,double c1,double c2,double V_Max)
	{
		vector<double> gBest;
		gBest=findGlobalBest(particles,numberOfParticles);
			for(int j=0;j<numberOfParticles;j++)
			{
				FindLocalBest(particles[j].getEntireElement(),particles[j],observations[j]);
				particles[j].setGlobalBest(gBest);
				updateVelocity(particles[j],dimension,w,c1,c2,V_Max);
				
			}
			
	}
