#include "RandomVariates.h"
#include "PSOFunctions.h"
#include<vector>
#include<string>
#include<fstream>
#include <sstream>




int main()
{
	/*
	*****************This code is for Standard PSO*******************

	In order to run the code, one first should write his/her own test function in "double objectiveValueWithoutNoise(vector<double> &newVector,int functionNumber)" function existing in the "ContinuousParticle.cpp".
	Right now, the ackley function is used as a sample to show how to write the function. Please follow the same procedure to write your own function.
	In the following lines, you can change and set the parameters you need to use for the algorithm. The explanations about each parameter is written above it.
	Afte setting the test function and parameters, the code is ready to run by pressing "ctrl+F5"

	The output of the algorithm is printed into a csv file named "results.csv". The output contains the mean and variance of the performance of algorithm at each iteration.

	*/

	/*********************************************************
	**********************************************************/
	//The dimension of test function should be set here
	int dimension=10;

	//Number of Macro-replications of PSO can be set here
	int numberOfIterations=100;

	//The variance of the guassian noise can be set here (The mean is 0)
	double noiseVariance=4;

	//Number of particles can be set here (default for the experiments is 50)
	const int numberOfParticles=50;

	//you can change the PSO parameters here
	double Budget=200; //The maximum number of PSO iterations we want to run at each iteration

	//PSO parameters
	double w=0.729,c1=1.49445,c2=1.49445, VMax=10;//w, c1,c2 are the actual parameters for PSO. VMax is the max velocity which can be set according to the test function

	//The number of observations on each particle:
	int numberOfObservations=5;
	/*********************************************************
	*********************************************************/

	//No changes are needed from here to the end of the code
	ofstream myfile;
	int temp;
	int flag=0;
	double sum=0;
	double a=0;
	double b0=0;
	Particle particles[numberOfParticles];
	ifstream input;
	double temp3;
	vector<vector<double>> initial;
	vector<vector<double>> observations;
	for(int i=0;i<numberOfParticles;i++)
	{
		observations.push_back(vector<double>());
	}

	//We read the random numbers from a text file in order to have a same set of random numbers for all iterations.
	input.open("random.txt");
	for(int i=0;i<numberOfParticles;i++)
	{
		initial.push_back(vector<double>());
		for(int j=0;j<dimension;j++)
		{
			input>>temp3;
			initial[i].push_back(temp3);
		}
	}
	input.close();
	vector<vector<double>> results;
	vector<double> tempVector;
	vector<double> tempGbest;
	//This for loop is for having 100 macroreplications which means that we do the standard PSO 100 times and we use the sample mean as an indicator of the performance
	for(int p=0;p<numberOfIterations;p++)
	{
		cout<<"iteration: "<<p+1<<endl;
		results.push_back(vector<double>());
		w=0.5,c1=1,c2=1, VMax=100;
	for(int i=0;i<Budget;i++)
	{
		observations.clear();
		observations.resize(0);
		observations.resize(numberOfParticles);
		
		
		
		if(i==0)
		{
			//The initialization of particles
		for(int r=0;r<numberOfParticles;r++)
	{
		particles[r].initializeVector(initial[r]);
	}
		for(int k=0;k<numberOfParticles;k++)
		{
			a=objectiveValueWithoutNoise(particles[k].getEntireElement());
			for(int j=0;j<numberOfObservations;j++)
			{
				srand(time(NULL));

				//regular
				observations[k].push_back(a+Normal(0,noiseVariance));

				//proportional
				//observations[k].push_back(a+Normal(0,sqrt(a-b0)));
			}
		}
		for(int k=0;k<numberOfParticles;k++)
		{
			particles[k].setlBestValue(CalculateSampleMean(observations[k]));
		}
		tempGbest=findGlobalBest(particles,numberOfParticles);
		for(int j=0;j<numberOfParticles;j++)
	{
		
			particles[j].setGlobalBest(tempGbest);
			
	}
		
		}
		else
		{
			for(int k=0;k<numberOfParticles;k++)
		{
			particles[k].setlBestValue(NoisyObjectiveValue(particles[k].getEntireLBest(),numberOfObservations,noiseVariance));
		}
			for(int k=0;k<numberOfParticles;k++)
			{
				a=objectiveValueWithoutNoise(particles[k].getEntireElement());
				for(int j=0;j<numberOfObservations;j++)
				{
					//regular:
					observations[k].push_back(a+Normal(0,noiseVariance));

					//proportional
					//observations[k].push_back(a+Normal(0,sqrt(a-b0)));
				}
			}
		}
		

		//The traditional PSO function
		PSO(particles,Budget,numberOfParticles,observations,dimension,w,c1,c2,VMax);

		//We save the results in a vector to calculate sample mean and variance at the end
		results[p].push_back(objectiveValueWithoutNoise(particles[0].getGlobalBest()));
		
	}
	}

	

	//We print the output in a csv file 
  myfile.open ("results.csv");
  myfile<<"Iteration"<<","<<"Mean"<<","<<"Variance"<<"\n";
	  for(int j=0;j<Budget;j++)
	  {
		  tempVector.clear();
		  tempVector.resize(0);
		  for(int i=0;i<numberOfIterations;i++)
		  {
			  tempVector.push_back(results[i][j]);
		  }
		  //Sample mean and sample variance will be saved in results.csv
		  myfile<<j+1<<","<<CalculateSampleMean(tempVector)<<","<<CalculateSampleVariance(tempVector)<<"\n";
	  }
  myfile.close();
  
  
	cin.get();
	

	return 0;
}